﻿using System;
using System.Collections.Generic;
using System.Linq;
using System.Text;

namespace タイマーくん
{
    /// <summary>
    /// タイマー情報クラス
    /// </summary>
    public class TimerInfo
    {
        /// <summary>
        /// [定数定義]
        /// </summary>
        public enum CONTROL_STATUS { Set, Reset, toggle }
        public enum KIND_REPETITION { なし, 曜日, 間隔 }
        public enum SPAN_UNIT { 秒, 分, 時間, 日 }
        /// <summary>
        /// [外部変数定義]
        /// </summary>
        public string Name = "";
        public byte Id = 0;
        public CONTROL_STATUS ControlStatus = CONTROL_STATUS.Set;
        public DateTime ControlTime = DateTime.Now;
        public KIND_REPETITION KindRepetition = KIND_REPETITION.なし;
        public bool[] Weekly = new bool[] { false, false, false, false, false, false, false };
        public long SpanCount = 0;
        public SPAN_UNIT SpanUnit = SPAN_UNIT.日;
        /// <summary>
        /// [コンストラクタ]
        /// </summary>
        public TimerInfo(){}
        /// <summary>
        /// プロパティ
        /// </summary>
        public string 名称 { get { return Name; } }
        public string デバイスID { get { return Id.ToString("X2"); } }
        public string 時刻 { get { return ControlTime.ToString("yyy/MM/dd HH:mm:ss"); } }
        public CONTROL_STATUS 制御 { set { ControlStatus = value; } get { return ControlStatus; } }
        public string 繰返し
        {
            get
            {
                string txt = "";
                switch (KindRepetition)
                {
                    case KIND_REPETITION.なし: txt = "なし"; break;
                    case KIND_REPETITION.曜日:
                        if (Weekly[0]) txt += "日";
                        if (Weekly[1]) txt += "月";
                        if (Weekly[2]) txt += "火";
                        if (Weekly[3]) txt += "水";
                        if (Weekly[4]) txt += "木";
                        if (Weekly[5]) txt += "金";
                        if (Weekly[6]) txt += "土";
                        break;
                    case KIND_REPETITION.間隔:
                        switch (SpanUnit)
                        {
                            case SPAN_UNIT.日: txt = string.Format("{0}日毎", SpanCount); break;
                            case SPAN_UNIT.時間: txt = string.Format("{0}時間毎", SpanCount); break;
                            case SPAN_UNIT.分: txt = string.Format("{0}分毎", SpanCount); break;
                            case SPAN_UNIT.秒: txt = string.Format("{0}秒毎", SpanCount); break;
                        }
                        break;
                }
                return txt;
            }
        }
        /// <summary>
        /// [メソッド]曜日の繰り返し取得
        /// [説明]現在設定されている日付けに対して繰返し対象曜日に当てはまるか否か、チェックします。
        /// </summary>
        /// <returns>true:繰返し対象曜日,false:繰返し対象外</returns>
        public bool RepetitionWeek()
        {
            switch (ControlTime.DayOfWeek)
            {
                case DayOfWeek.Sunday: return (Weekly[0]);
                case DayOfWeek.Monday: return (Weekly[1]);
                case DayOfWeek.Tuesday: return (Weekly[2]);
                case DayOfWeek.Wednesday: return (Weekly[3]);
                case DayOfWeek.Thursday: return (Weekly[4]);
                case DayOfWeek.Friday: return (Weekly[5]);
                case DayOfWeek.Saturday: return (Weekly[6]);
            }
            return false;
        }
    }
}
