﻿using System;
using System.Collections.Generic;
using System.ComponentModel;
using System.Data;
using System.Drawing;
using System.Linq;
using System.Text;
using System.Windows.Forms;
using System.Xml.Serialization;
using System.IO;
using System.Threading;
using System.Security.Permissions;

namespace タイマーくん
{
    public partial class frmMain : Form
    {
        private string _ini_file_name = Application.LocalUserAppDataPath + @"\タイマーくん.xml";
        private List<TimerInfo> _timer_infos = new List<TimerInfo>();
        private Dictionary<byte, bool> _init = new Dictionary<byte, bool>();
        private BindingSource _bs = new BindingSource();
        private Thread _control = null; 
        private bool _balloon = false;
        private bool _force_close = false;  /* 強制クローズ */
        /// <summary>
        /// デリゲート
        /// </summary>
        delegate void del_redraw_grd();                         /* グリッド描画 */
        delegate void del_append_log(string err, string msg);   /* ログ追記 */

        /// <summary>
        /// [コンストラクタ]
        /// </summary>
        public frmMain()
        {
            InitializeComponent();
            frmAbout f = new frmAbout(true);
            f.ShowDialog();
            timTime.Enabled = true; /* 時計表示の開始 */
            _read_ini_file();       /* 初期設定ファイル読み込み */
            _init_grid();           /* グリッドの初期化 */
            _start_control();       /* 制御スレッド開始 */

        }
        /// <summary>
        /// [イベント]フォームのクローズ中
        /// </summary>
        /// <param name="sender"></param>
        /// <param name="e"></param>
        private void frmMain_FormClosing(object sender, FormClosingEventArgs e)
        {
            if (!_force_close)  /* 強制クローズでなければフォーム隠し */
            {
                e.Cancel = true;
                this.Hide();
            }
            else
            {
                _write_ini_file();  /* 現在の情報をファイルに書き込み */
            }
        }
        /// <summary>
        /// 時計表示タイマー
        /// </summary>
        /// <param name="sender"></param>
        /// <param name="e"></param>
        private void timTime_Tick(object sender, EventArgs e)
        {
            string s = DateTime.Now.ToString("yyyy/MM/dd HH:mm:ss");
            if (lblTimer.Text != s) lblTimer.Text = s;
        }
        /// <summary>
        /// [イベント][ファイル]-[終了]メニュー
        /// </summary>
        /// <param name="sender"></param>
        /// <param name="e"></param>
        private void mnuExit_Click(object sender, EventArgs e)
        {
            DialogResult result = MessageBox.Show("タイマー制御を終了します。\r\nよろしいですか？", Text, MessageBoxButtons.YesNo, MessageBoxIcon.Question);
            if (result == DialogResult.No) return;
            _stop_control();            /* 制御の停止 */
            timTime.Enabled = false;    /* 時計表示の停止 */
            _all_close_rlsw();          /* デバイスのクローズ */
            _force_close = true;        /* 強制フラグON */
            this.Close();
        }
        /// <summary>
        /// [イベント][タイマ]-[新規作成]メニュー
        /// </summary>
        /// <param name="sender"></param>
        /// <param name="e"></param>
        private void mnuNew_Click(object sender, EventArgs e)
        {
            TimerInfo info = new TimerInfo();
            frmSetting f = new frmSetting(info);
            if (f.ShowDialog() == DialogResult.OK)  /* タイマー編集画面表示 */
            {
                _bs.Add(info);  /* タイマー情報追加 */
            }
        }
        /// <summary>
        /// [イベント][タイマ]-[編集]メニュー
        /// </summary>
        /// <param name="sender"></param>
        /// <param name="e"></param>
        private void mnuEdit_Click(object sender, EventArgs e)
        {
            if (grdTimers.SelectedRows.Count == 0) return;  /* 選択されてないとやらない */
            TimerInfo info = (TimerInfo)_bs.Current;
            if (info == null) return;
            frmSetting f = new frmSetting(info);
            if (f.ShowDialog() == DialogResult.OK)  /* タイマー編集画面表示 */
            {
                _bs.ResetItem(_bs.Position);    /* タイマー情報更新 */
            }
        }
        /// <summary>
        /// [イベント][タイマ]-[削除]メニュー
        /// </summary>
        /// <param name="sender"></param>
        /// <param name="e"></param>
        private void mnuDelete_Click(object sender, EventArgs e)
        {
            if (grdTimers.SelectedRows.Count == 0) return;  /* 選択されてないとやらない */
            TimerInfo info = (TimerInfo)_bs.Current;
            if (info == null) return;
            DialogResult result = MessageBox.Show(string.Format("タイマー[{0}]を削除します。よろしいですか？", info.Name), Text, MessageBoxButtons.YesNo, MessageBoxIcon.Question);
            if (result == DialogResult.No) return;
            _bs.RemoveCurrent();    /* タイマー情報削除 */
        }
        /// <summary>
        /// [イベント][表示]-[ログのクリア]メニュー
        /// </summary>
        /// <param name="sender"></param>
        /// <param name="e"></param>
        private void mnuClearLog_Click(object sender, EventArgs e)
        {
            txtLog.Clear(); /* ログ表示の消去 */
        }
        /// <summary>
        /// [イベント][表示]-[バルーン表示]メニュー
        /// </summary>
        /// <param name="sender"></param>
        /// <param name="e"></param>
        private void mnuBalloon_Click(object sender, EventArgs e)
        {
            mnuBalloon.Checked = !mnuBalloon.Checked;   /* バルーン表示有無の切り替え */
            _balloon = mnuBalloon.Checked;
        }
        /// <summary>
        /// [イベント][ヘルプ]-[バージョン]メニュー
        /// </summary>
        /// <param name="sender"></param>
        /// <param name="e"></param>
        private void mnuVersion_Click(object sender, EventArgs e)
        {
            frmAbout f = new frmAbout(false);
            f.ShowDialog(); /* バージョン画面表示 */
        }
        /// <summary>
        /// [イベント][コンテキストメニュー]-[設定]
        /// </summary>
        /// <param name="sender"></param>
        /// <param name="e"></param>
        private void mnuSet_Click(object sender, EventArgs e)
        {
            this.Show();    /* メイン画面の表示 */
        }
        /// <summary>
        /// [イベント]グリッドセルのダブルクリック
        /// </summary>
        /// <param name="sender"></param>
        /// <param name="e"></param>
        private void grdTimers_CellContentDoubleClick(object sender, DataGridViewCellEventArgs e)
        {
            mnuEdit_Click(sender, new EventArgs()); /* 編集メニューが押されたのと同じ */
        }
        /// <summary>
        /// [イベント]グリッドセルのキー入力
        /// </summary>
        /// <param name="sender"></param>
        /// <param name="e"></param>
        private void grdTimers_KeyDown(object sender, KeyEventArgs e)
        {
            switch (e.KeyData)
            {
                case Keys.Delete:   /* 削除キー */
                    mnuDelete_Click(sender, new EventArgs());   /* 削除メニューが押されたのと同じ */
                    break;
                case Keys.Return:   /* リターンキー */
                    mnuEdit_Click(sender, new EventArgs());     /* 編集メニューが押されたのと同じ */
                    break;
            }
        }
        /// <summary>
        /// [関数]初期設定ファイルの読み込み
        /// [説明]XMLファイルに書き込まれたタイマ情報の読み込み
        /// </summary>
        private void _read_ini_file()
        {
            if (File.Exists(_ini_file_name))
            {
                XmlSerializer serializer = new XmlSerializer(typeof(List<TimerInfo>));
                StreamReader sr = new StreamReader(_ini_file_name, new UTF8Encoding(false));
                try
                {
                    _timer_infos = (List<TimerInfo>)serializer.Deserialize(sr);
                }
                catch
                {
                    MessageBox.Show("設定ファイルの読み込み異常が発生しました。\r\n初期状態で起動します。", Text, MessageBoxButtons.OK, MessageBoxIcon.Error);
                }
                sr.Close();
            }
        }
        /// <summary>
        /// [関数]初期設定ファイルの書き込み
        /// [説明]XMLファイルにタイマ情報を書き込み
        /// </summary>
        private void _write_ini_file()
        {
            Directory.CreateDirectory(Path.GetDirectoryName(_ini_file_name));
            XmlSerializer serializer = new XmlSerializer(typeof(List<TimerInfo>));
            StreamWriter sw = new StreamWriter(_ini_file_name, false, new UTF8Encoding(false));
            serializer.Serialize(sw, _timer_infos);
            sw.Close();
        }
        /// <summary>
        /// [関数]グリッドの初期化
        /// [説明]グリッドの体裁整え
        ///       タイマー情報表示
        /// </summary>
        private void _init_grid()
        {
            //体裁整え
            grdTimers.BorderStyle = BorderStyle.None;
            grdTimers.AllowUserToAddRows = false;
            grdTimers.AllowUserToDeleteRows = false;
            grdTimers.AllowUserToResizeRows = false;
            grdTimers.ColumnHeadersVisible = false;
            grdTimers.RowHeadersVisible = false;
            grdTimers.SelectionMode = DataGridViewSelectionMode.FullRowSelect;
            //タイマー情報表示
            _bs.DataSource = _timer_infos;
            grdTimers.DataSource = _bs;
            //体裁整え
            grdTimers.Columns["名称"].Width = 80;
            grdTimers.Columns["デバイスID"].Width = 25;
            grdTimers.Columns["時刻"].Width = 140;
            grdTimers.Columns["制御"].Width = 60;
            grdTimers.Columns["繰返し"].Width = 120;
            grdTimers.Columns["デバイスID"].DefaultCellStyle.Alignment = DataGridViewContentAlignment.MiddleRight;
            grdTimers.Columns["時刻"].DefaultCellStyle.Alignment = DataGridViewContentAlignment.MiddleCenter;
            grdTimers.Columns["制御"].DefaultCellStyle.Alignment = DataGridViewContentAlignment.MiddleCenter;
            grdTimers.Columns["繰返し"].DefaultCellStyle.Alignment = DataGridViewContentAlignment.MiddleCenter;
        }
        /// <summary>
        /// [関数]制御スレッドの起動
        /// </summary>
        private void _start_control()
        {
            _control = new Thread(new ThreadStart(_thread_control));
            _control.Start();
        }
        /// <summary>
        /// [関数]制御スレッドの停止
        /// </summary>
        private void _stop_control()
        {
            _control.Abort();
            _control.Join();
        }
        /// <summary>
        /// [関数]制御スレッド
        /// </summary>
        private void _thread_control()
        {
            DateTime time_last = DateTime.Now;
            DateTime time_now;
            bool init;
            bool draw = false;
            while (true)
            {
                time_now = DateTime.Now;
                draw = false;
                foreach (TimerInfo s in _timer_infos)
                {
                    if (_timer_forward(s, time_last)) draw = true;
                    if (time_last <= s.ControlTime && s.ControlTime <= time_now)
                    {
                        init = false;
                        draw = true;
                        if (_init.ContainsKey(s.Id))
                        {
                            if (!_init[s.Id]) init = true;
                        }
                        else
                        {
                            _init.Add(s.Id, false);
                            init = true;
                        }
                        if (init)
                        {
                            if (sacomUsbIoFt.sacomUsbIoOpen(s.Id)) _init[s.Id] = true;
                        }
                        if (!_control_rlsw(s)) _init[s.Id] = false; /* リレー制御に失敗したら初期化からやり直す */
                    }
                }
                time_last = time_now;
                if (draw) Invoke(new del_redraw_grd(_redraw_grd));
                Thread.Sleep(100);
            }
        }
        /// <summary>
        /// [関数]繰返しタイマーの時刻再設定
        /// [説明]繰返し設定がされているタイマー情報において、次回の制御時刻の再設定を行う。
        /// </summary>
        /// <param name="s">タイマー情報</param>
        /// <param name="last">前回処理時間</param>
        /// <returns>true:再設定あり,false:再設定なし</returns>
        private bool _timer_forward(TimerInfo s, DateTime last)
        {
            bool result = false;

            switch (s.KindRepetition)
            {
                case TimerInfo.KIND_REPETITION.なし: return (result);  /* 繰返し無し設定時は何もしない */
                case TimerInfo.KIND_REPETITION.間隔:
                    while (s.ControlTime < last)   /* 再設定の必要あり？(現在設定されている時刻が過ぎている) */
                    {
                        switch (s.SpanUnit)
                        {
                            case TimerInfo.SPAN_UNIT.日:
                                s.ControlTime = s.ControlTime.AddDays(s.SpanCount); break; 
                            case TimerInfo.SPAN_UNIT.時間:
                                s.ControlTime = s.ControlTime.AddHours(s.SpanCount); break;
                            case TimerInfo.SPAN_UNIT.分:
                                s.ControlTime = s.ControlTime.AddMinutes(s.SpanCount); break;
                            case TimerInfo.SPAN_UNIT.秒:
                                s.ControlTime = s.ControlTime.AddSeconds(s.SpanCount); break;
                        }
                        result = true;  /* 更新あり */
                    }
                    break;
                case TimerInfo.KIND_REPETITION.曜日:
                    while (s.ControlTime < last || !s.RepetitionWeek())
                    {
                        s.ControlTime = s.ControlTime.AddDays(1);   //次の日に移動
                        result = true;  /* 更新あり */
                    }
                    break;
            }
            return result;
        }
        /// <summary>
        /// [関数]IO出力の制御
        /// [説明]タイマー情報に従って点灯/消灯/反転を行います。
        /// </summary>
        /// <param name="info">タイマー情報</param>
        /// <returns>true:制御成功,false:制御失敗</returns>
        private bool _control_rlsw(TimerInfo info)
        {
            bool status = false;
            DateTime tim = DateTime.Now;

            switch (info.ControlStatus)
            {
                case TimerInfo.CONTROL_STATUS.Set: status = true; break;
                case TimerInfo.CONTROL_STATUS.Reset: status = false; break;
                case TimerInfo.CONTROL_STATUS.toggle:
                    Int32 i = 0;
                    sacomUsbIoFt.sacomUsbIoReadback(info.Id, ref i);
                    status = ((i & 1) == 0);
                    break;
            }
            if (sacomUsbIoFt.sacomUsbIoBit(info.Id, 0, status))
            {
                if (status)
                {
                    _log_push(info, "Set", "");
                }
                else
                {
                    _log_push(info, "Reset", "");
                }
                return true;
            }
            else
            {
                if (status)
                {
                    _log_push(info, "Set", "sacomUsbIoBit()");
                }
                else
                {
                    _log_push(info, "Reset", "sacomUsbIoBit()");
                }
                return false;
            }
        }
        /// <summary>
        /// タイマー情報に登録されているデバイスを終了させる
        /// </summary>
        private void _all_close_rlsw()
        {
            foreach (TimerInfo s in _timer_infos)
            {
                sacomUsbIoFt.sacomUsbIoClose(s.Id);
            }
        }
        /// <summary>
        /// [関数]グリッドの再描画
        /// </summary>
        private void _redraw_grd()
        {
            grdTimers.Refresh();
        }
        /// <summary>
        /// [関数]ログ追記処理
        /// [説明]制御ログ全般を処理します。
        /// </summary>
        /// <param name="info"></param>
        /// <param name="status"></param>
        /// <param name="err"></param>
        private void _log_push(TimerInfo info, string status,string err)
        {
            _balloon_push(info, status, err);
            string txt = (err == "") ? string.Format("{0} {1}[{2}]:{3}\r\n", DateTime.Now.ToString("HH:mm:ss"), info.Name, info.デバイスID, status) :
                                       string.Format("{0} NG:{1}[{2}]:{3})\r\n", DateTime.Now.ToString("HH:mm:ss"), info.Name, info.デバイスID, err);
            Invoke(new del_append_log(_append_log),err,txt);
        }
        /// <summary>
        /// [関数]ログバルーン表示
        /// 制御ログをバルーンヘルプで表示します。
        /// </summary>
        /// <param name="info"></param>
        /// <param name="status"></param>
        /// <param name="err"></param>
        private void _balloon_push(TimerInfo info, string status, string err)
        {
            if (!_balloon) return;
            nicIcon.BalloonTipTitle = info.Name;
            string txt = (err == "") ? string.Format("{0} {1}[{2}]:{3}\r\n", DateTime.Now.ToString("HH:mm:ss"), info.Name, info.デバイスID, status) :
                                       string.Format("{0} NG:{1}[{2}]:{3}\r\n", DateTime.Now.ToString("HH:mm:ss"), info.Name, info.デバイスID, err);
            nicIcon.BalloonTipText = txt;
            nicIcon.ShowBalloonTip(1000);
        }
        /// <summary>
        /// [関数]ログテキスト追記
        /// [説明]テキストボックスに追記します。
        /// </summary>
        /// <param name="err">エラー有無</param>
        /// <param name="msg">ログメッセージ</param>
        private void _append_log(string err, string msg)
        {
            txtLog.Focus();
            txtLog.SelectionLength = 0;
            txtLog.SelectionStart = txtLog.TextLength;
            txtLog.SelectionColor = (err == "") ? Color.Black : Color.Red;
            txtLog.SelectedText = msg;
        }
    }
}
