﻿using System;
using System.Collections.Generic;
using System.ComponentModel;
using System.Data;
using System.Drawing;
using System.Linq;
using System.Text;
using System.Windows.Forms;

namespace タイマーくん
{
    public partial class frmSetting : Form
    {
        private TimerInfo _set;
        private bool _cancel = false;

        /// <summary>
        /// [コンストラクタ]
        /// </summary>
        /// <param name="set">タイマー情報</param>
        public frmSetting(TimerInfo set)
        {
            InitializeComponent();
            //コントロールの設定
            _set = set;
            txtName.Text = set.名称;
            txtId.Text = set.デバイスID;
            cboControlStatus.DataSource = Enum.GetValues(typeof(TimerInfo.CONTROL_STATUS));
            cboControlStatus.Text = set.制御.ToString();
            dtpTime.Value = set.ControlTime;
            cboUnit.DataSource = Enum.GetValues(typeof(TimerInfo.SPAN_UNIT));
            switch (set.KindRepetition)
            {
                case TimerInfo.KIND_REPETITION.なし:
                    grpRepetition.Enabled = false;
                    chkRepetition.Checked = false;
                    break;
                case TimerInfo.KIND_REPETITION.曜日:
                    grpRepetition.Enabled = true;
                    chkRepetition.Checked = true;
                    grpWeek.Enabled = true;
                    rdoWeek.Checked = true;
                    chkSun.Checked = set.Weekly[0];
                    chkMon.Checked = set.Weekly[1];
                    chkTue.Checked = set.Weekly[2];
                    chkWed.Checked = set.Weekly[3];
                    chkThu.Checked = set.Weekly[4];
                    chkFri.Checked = set.Weekly[5];
                    chkSta.Checked = set.Weekly[6];
                    grpSpan.Enabled = false;
                    rdoSpan.Checked = false;
                    break;
                case TimerInfo.KIND_REPETITION.間隔:
                    grpRepetition.Enabled = true;
                    chkRepetition.Checked = true;
                    grpSpan.Enabled = true;
                    rdoSpan.Checked = true;
                    txtSpan.Text = set.SpanCount.ToString();
                    cboUnit.Text = set.SpanUnit.ToString();
                    grpWeek.Enabled = false;
                    rdoWeek.Checked = false;
                    break;
            }
        }
        /// <summary>
        /// [イベント]フォームのクローズ
        /// </summary>
        /// <param name="sender"></param>
        /// <param name="e"></param>
        private void frmSetting_FormClosing(object sender, FormClosingEventArgs e)
        {
            e.Cancel = _cancel;
            _cancel = false;
        }
        /// <summary>
        /// [イベント]繰返しチェックボックスの変更
        /// </summary>
        /// <param name="sender"></param>
        /// <param name="e"></param>
        private void chkRepetition_CheckedChanged(object sender, EventArgs e)
        {
            grpRepetition.Enabled = chkRepetition.Checked;
        }
        /// <summary>
        /// [イベント]曜日ラジオボタンの変更
        /// </summary>
        /// <param name="sender"></param>
        /// <param name="e"></param>
        private void rdoWeek_CheckedChanged(object sender, EventArgs e)
        {
            grpWeek.Enabled = rdoWeek.Checked;
            grpSpan.Enabled = rdoSpan.Checked;
        }
        /// <summary>
        /// [イベント]OKボタンのクリック
        /// </summary>
        /// <param name="sender"></param>
        /// <param name="e"></param>
        private void btnOk_Click(object sender, EventArgs e)
        {
            byte id;
            long count;
            int i;
            TimerInfo tmp = new TimerInfo();

            _cancel = false;
            txtName.Text = txtName.Text.Trim();
            tmp.Name = txtName.Text;
            txtId.Text = txtId.Text.Trim();
            try
            {
                id = Convert.ToByte(txtId.Text,16);
                if ((byte)id == 0xff)
                {
                    MessageBox.Show("IDの設定値が正しくありません。\r\n00～FE");
                    _cancel = true;
                }
                tmp.Id = id;
            }
            catch
            {
                MessageBox.Show("IDの設定値が正しくありません。");
                _cancel = true;
            }
            tmp.ControlTime = dtpTime.Value;
            tmp.ControlStatus = (TimerInfo.CONTROL_STATUS)Enum.Parse(typeof(TimerInfo.CONTROL_STATUS),cboControlStatus.SelectedItem.ToString());
            if (chkRepetition.Checked)
            {
                if (rdoWeek.Checked)
                {
                    tmp.KindRepetition = TimerInfo.KIND_REPETITION.曜日;
                    tmp.Weekly[0] = chkSun.Checked;
                    tmp.Weekly[1] = chkMon.Checked;
                    tmp.Weekly[2] = chkTue.Checked;
                    tmp.Weekly[3] = chkWed.Checked;
                    tmp.Weekly[4] = chkThu.Checked;
                    tmp.Weekly[5] = chkFri.Checked;
                    tmp.Weekly[6] = chkSta.Checked;
                    for (i = 0; i < 7; i++)
                    {
                        if (tmp.Weekly[i]) break;
                    }
                    if (i == 7)
                    {
                        MessageBox.Show("最低１つの曜日を設定して下さい。");
                        _cancel = true;
                    }
                }
                else
                {
                    tmp.KindRepetition = TimerInfo.KIND_REPETITION.間隔;
                    txtSpan.Text = txtSpan.Text.Trim();
                    if (!long.TryParse(txtSpan.Text, out count))
                    {
                        MessageBox.Show("間隔の設定値が正しくありません。");
                        _cancel = true;
                    }
                    else
                    {
                        tmp.SpanCount = count;
                    }
                    tmp.SpanUnit = (TimerInfo.SPAN_UNIT)Enum.Parse(typeof(TimerInfo.SPAN_UNIT), cboUnit.SelectedItem.ToString());
                }
            }
            else
            {
                tmp.KindRepetition = TimerInfo.KIND_REPETITION.なし;
            }
            if (!_cancel)
            {
                _set.Name = tmp.Name;
                _set.Id = tmp.Id;
                _set.ControlTime = tmp.ControlTime;
                _set.ControlStatus = tmp.ControlStatus;
                _set.KindRepetition = tmp.KindRepetition;
                _set.Weekly = tmp.Weekly;
                _set.SpanCount = tmp.SpanCount;
                _set.SpanUnit = tmp.SpanUnit;
            }
        }
    }
}
